//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#include "ez_ilda.hpp"

//############################################################################
void spokes(ez_ilda_frame_set* p_frame_set)
{
    ez_ilda_frame_set  out;
    int                spoke;
    ez_3D_float        a(0, 0, pi/6);

    for(int frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {
        out += p_frame_set->at(frame_count);
        for(spoke = 1; spoke < 12; spoke++)
        {
            out[frame_count].rotate_around_origin(a);
            out[frame_count] += p_frame_set->at(frame_count);
        }
        out[frame_count].rotate_colors();
    }
    out.save_as_ilda("spokes.ild");
    return;
}

//############################################################################
void gyro(ez_ilda_frame_set* p_frame_set)
{
    ez_3D_float        a(one_degree, one_degree, one_degree);
    ez_ilda_frame      frame;
    ez_ilda_frame_set  out;

    for(int frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {
        frame = p_frame_set->at(frame_count);
        frame.rotate_around_origin(a * frame_count);
        out += frame;
    }
    out.save_as_ilda("gyro.ild");
    return;
}

//############################################################################
void echos(ez_ilda_frame_set* p_frame_set)
{
    int   i,
          echo,
          delay     =  1,
          num_echos = 10;

    ez_ilda_frame_set  out;
    ez_ilda_frame      empty_frame(p_frame_set->at(0).palette_index, 1); // not really!

    for(i = 0; i < (p_frame_set->number_of_frames() + (delay * num_echos)); i++)
    {
        if(i < p_frame_set->number_of_frames())
            out += p_frame_set->at(i);
        else
            out += empty_frame;

        for(echo = 1; echo <= num_echos; echo++)
            if(    i >= (delay * echo)
                && (i - delay * echo < p_frame_set->number_of_frames())
              )
                out[i] += p_frame_set->at(i - delay * echo);
    }
    out.save_as_ilda("echos.ild");
    return;
}

//############################################################################
void gears(ez_ilda_frame_set* p_frame_set)
{ // requires a frame set of 4 or more frames. (Try in.ild)
    if(p_frame_set->number_of_frames() > 3)
    {
        ez_ilda_frame_set  out;
        
        ez_ilda_frame g20 = p_frame_set->at(1);
        ez_ilda_frame g16 = p_frame_set->at(2);
        ez_ilda_frame g12 = p_frame_set->at(3);

        g20.scale_around_origin(ez_3D_float(0.500, 0.500, 0.500));
        g16.scale_around_origin(ez_3D_float(0.375, 0.375, 0.375));
        g12.scale_around_origin(ez_3D_float(0.300, 0.300, 0.300));

        g20.move(ez_3D_short(-10000,      0, 0));
        g16.move(ez_3D_short( 12200, -15400, 0));
        g12.move(ez_3D_short(  5150,  19300, 0));
                    
        out += (g20 + g16 + g12);

        ez_ilda_frame f20 = g20;
        ez_ilda_frame f16 = g16;
        ez_ilda_frame f12 = g12;

        for(int i = 1; i < 480; i++)
        {
            f20 = g20;
            f16 = g16;
            f12 = g12;

            f20.rotate(ez_3D_float(0, 0, -pi/60) * i);
            f16.rotate(ez_3D_float(0, 0,  pi/48) * i);
            f12.rotate(ez_3D_float(0, 0,  pi/36) * i);

            out += (f20 + f16 + f12);
        }
        out.save_as_ilda("gears.ild");
    }
    return;
}

//############################################################################
void vortex(ez_ilda_frame_set* p_frame_set)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame;
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.80, 0.80, 0.80);
    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 19; shrink_frame_count++)
        {        
            out += frame;
            frame.scale_around_origin(s);
        }
    }
    out.save_as_ilda("vortex.ild");
}

//############################################################################
void drain(ez_ilda_frame_set* p_frame_set)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame;
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.85, 0.85, 0.85),
                       a(0.0,  0.0,  pi/15.0);
    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 29; shrink_frame_count++)
        {        
            out += frame;
            frame.scale_around_origin(s);
            frame.rotate_around_origin(a);
        }
    }
    out.save_as_ilda("drain.ild");
}

//############################################################################
void diminish(ez_ilda_frame_set* p_frame_set)
{
    ez_ilda_frame_set  out;
    ez_ilda_frame      frame;
    int                frame_count,
                       shrink_frame_count;
    ez_3D_float        s(0.75, 0.75, 0.75);
    for(frame_count = 0; frame_count < p_frame_set->number_of_frames(); frame_count++)
    {        
        frame = p_frame_set->at(frame_count);
        for(shrink_frame_count = 0; shrink_frame_count < 19; shrink_frame_count++)
        {        
            out += frame;
            frame.scale(s);
        }
    }
    out.save_as_ilda("diminish.ild");
}

//############################################################################







//############################################################################
void ez_ilda::load_frame_set_effects()
{
    frame_set_effects.push_back(spokes);
    frame_set_effect_names.push_back(string("spokes"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(gyro);
    frame_set_effect_names.push_back(string("gyro"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(echos);
    frame_set_effect_names.push_back(string("echos"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(gears);
    frame_set_effect_names.push_back(string("gears"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(vortex);
    frame_set_effect_names.push_back(string("vortex"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(drain);
    frame_set_effect_names.push_back(string("drain"));
    //------------------------------------------------------------------------
    frame_set_effects.push_back(diminish);
    frame_set_effect_names.push_back(string("diminish"));
    //------------------------------------------------------------------------
    return;
}

//############################################################################
